@@ -42,10 +42,6 @@ extern "C" {
 #  define mrn_override
 #endif
 
-#if (MYSQL_VERSION_ID >= 50514 && MYSQL_VERSION_ID < 50600)
-#  define MRN_HANDLER_HAVE_FINAL_ADD_INDEX 1
-#endif
-
 #if (MYSQL_VERSION_ID >= 50603) || defined(MRN_MARIADB_P)
 #  define MRN_HANDLER_HAVE_HA_RND_NEXT 1
 #  define MRN_HANDLER_HAVE_HA_RND_POS 1
@@ -81,10 +77,6 @@ extern "C" {
 #  define MRN_HANDLER_HAVE_MULTI_RANGE_READ_INFO_KEY_PARTS
 #endif
 
-#if MYSQL_VERSION_ID < 50600
-#  define MRN_HANDLER_HAVE_GET_TABLESPACE_NAME
-#endif
-
 #if MYSQL_VERSION_ID >= 50607
 #  define MRN_HANDLER_HAVE_SET_HA_SHARE_REF
 #endif
@@ -173,10 +165,6 @@ extern "C" {
 #  define MRN_TIMESTAMP_USE_LONG
 #endif
 
-#if MYSQL_VERSION_ID < 50600 && !defined(MRN_MARIADB_P)
-#  define MRN_FIELD_STORE_TIME_NEED_TYPE
-#endif
-
 #if MYSQL_VERSION_ID < 50706 || defined(MRN_MARIADB_P)
 #  define MRN_HAVE_TL_WRITE_DELAYED
 #endif
@@ -193,14 +181,6 @@ extern "C" {
 #  define MRN_JOIN_TAB_HAVE_CONDITION
 #endif
 
-#if MYSQL_VERSION_ID < 50600
-#  define MRN_RBR_UPDATE_NEED_ALL_COLUMNS
-#endif
-
-#if MYSQL_VERSION_ID >= 50500
-#  define MRN_ROW_BASED_CHECK_IS_METHOD
-#endif
-
 #if MYSQL_VERSION_ID >= 50600
 #  define MRN_HAVE_HA_REBIND_PSI
 #endif
@@ -220,16 +200,6 @@ extern "C" {
 #  endif
 #endif
 
-#ifdef MRN_MARIADB_P
-#  if MYSQL_VERSION_ID >= 50542 && MYSQL_VERSION_ID < 100000
-#    define MRN_SUPPORT_THDVAR_SET
-#  elif MYSQL_VERSION_ID >= 100017
-#    define MRN_SUPPORT_THDVAR_SET
-#  endif
-#else
-#  define MRN_SUPPORT_THDVAR_SET
-#endif
-
 #ifdef MRN_MARIADB_P
 #  if MYSQL_VERSION_ID < 100000
 #    define MRN_SUPPORT_PARTITION
@@ -405,6 +375,12 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_HAVE_GET_FOREIGN_DUP_KEY
 #endif
 
+#if !defined(MRN_MARIADB_P) ||                          \
+  (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID < 100500)
+#  define MRN_HANDLER_HAVE_NOTIFY_TABLE_CHANGED
+#  define MRN_HANDLER_HAVE_PRIMARY_KEY_IS_CLUSTERED
+#endif
+
 #if defined(HAVE_PSI_INTERFACE) &&                      \
   (MYSQL_VERSION_ID < 80002 || defined(MRN_MARIADB_P))
 #  define MRN_HAVE_PSI_SERVER
@@ -484,6 +460,10 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_HAVE_FOREIGN_KEY_INFO
 #endif
 
+#if MYSQL_VERSION_ID >= 100504 && defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_RECORDS_IN_RANGE_HAVE_PAGE_RANGE
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -557,9 +537,6 @@ private:
   mutable TABLE_SHARE table_share_for_create;
   mutable MEM_ROOT    mem_root_for_create;
   mutable handler     *wrap_handler_for_create;
-#ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
-  handler_add_index *hnd_add_index;
-#endif
 #ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
   mrn_alter_flags alter_handler_flags;
   KEY         *alter_key_info_buffer;
@@ -707,7 +684,16 @@ public:
 #endif
     ) const mrn_override;
 
-  ha_rows records_in_range(uint inx, key_range *min_key, key_range *max_key);
+#ifdef MRN_HANDLER_RECORDS_IN_RANGE_HAVE_PAGE_RANGE
+  ha_rows records_in_range(uint inx,
+                           const key_range *min_key,
+                           const key_range *max_key,
+                           page_range *pages) mrn_override;
+#else
+  ha_rows records_in_range(uint inx,
+                           key_range *min_key,
+                           key_range *max_key) mrn_override;
+#endif
   int index_init(uint idx, bool sorted);
   int index_end();
 #ifndef MRN_HANDLER_HAVE_HA_INDEX_READ_MAP
@@ -853,13 +839,7 @@ public:
                                    Alter_inplace_info *ha_alter_info);
 #else
   mrn_alter_table_flags alter_table_flags(mrn_alter_table_flags flags);
-#  ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
-  int add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
-                handler_add_index **add);
-  int final_add_index(handler_add_index *add, bool commit);
-#  else
   int add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
-#  endif
   int prepare_drop_index(TABLE *table_arg, uint *key_num, uint num_of_keys);
   int final_drop_index(TABLE *table_arg);
 #endif
@@ -915,11 +895,10 @@ protected:
 #ifdef MRN_HANDLER_HAVE_HA_INDEX_LAST
   int index_last(uchar *buf);
 #endif
+#ifdef MRN_HANDLER_HAVE_PRIMARY_KEY_IS_CLUSTERED
   bool primary_key_is_clustered()
     MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST
     mrn_override;
-#ifdef MRN_HANDLER_HAVE_GET_TABLESPACE_NAME
-  char *get_tablespace_name(THD *thd, char *name, uint name_len);
 #endif
 #ifdef MRN_HANDLER_HAVE_CAN_SWITCH_ENGINES
   bool can_switch_engines();
@@ -972,11 +951,13 @@ protected:
                                   dd::Table *new_table_def
 #  endif
     ) mrn_override;
+#  ifdef MRN_HANDLER_HAVE_NOTIFY_TABLE_CHANGED
   void notify_table_changed(
-#  ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
+#    ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
     Alter_inplace_info *ha_alter_info
-#  endif
+#    endif
     ) mrn_override;
+#  endif
 #endif
 
 private:
@@ -1406,12 +1387,22 @@ private:
   int storage_rnd_pos(uchar *buf, uchar *pos);
   void wrapper_position(const uchar *record);
   void storage_position(const uchar *record);
-  ha_rows wrapper_records_in_range(uint key_nr, key_range *range_min,
+#ifdef MRN_HANDLER_RECORDS_IN_RANGE_HAVE_PAGE_RANGE
+  ha_rows wrapper_records_in_range(uint key_nr,
+                                   const key_range *range_min,
+                                   const key_range *range_max,
+                                   page_range *pages);
+#else
+  ha_rows wrapper_records_in_range(uint key_nr,
+                                   key_range *range_min,
                                    key_range *range_max);
-  ha_rows storage_records_in_range(uint key_nr, key_range *range_min,
-                                   key_range *range_max);
-  ha_rows generic_records_in_range_geo(uint key_nr, key_range *range_min,
-                                       key_range *range_max);
+#endif
+  ha_rows storage_records_in_range(uint key_nr,
+                                   const key_range *range_min,
+                                   const key_range *range_max);
+  ha_rows generic_records_in_range_geo(uint key_nr,
+                                       const key_range *range_min,
+                                       const key_range *range_max);
   int wrapper_index_init(uint idx, bool sorted);
   int storage_index_init(uint idx, bool sorted);
   int wrapper_index_end();
@@ -1736,32 +1727,23 @@ private:
                                           dd::Table *new_table_def
 #  endif
     );
+#  ifdef MRN_HANDLER_HAVE_NOTIFY_TABLE_CHANGED
   void wrapper_notify_table_changed(
-#  ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
+#    ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
     Alter_inplace_info *ha_alter_info
-#  endif
+#    endif
     );
   void storage_notify_table_changed(
-#  ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
+#    ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
     Alter_inplace_info *ha_alter_info
-#  endif
+#    endif
     );
+#  endif
 #else
   mrn_alter_table_flags wrapper_alter_table_flags(mrn_alter_table_flags flags);
   mrn_alter_table_flags storage_alter_table_flags(mrn_alter_table_flags flags);
-#  ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
-  int wrapper_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
-                        handler_add_index **add);
-  int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys,
-                        handler_add_index **add);
-#  else
   int wrapper_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
   int storage_add_index(TABLE *table_arg, KEY *key_info, uint num_of_keys);
-#  endif
-#  ifdef MRN_HANDLER_HAVE_FINAL_ADD_INDEX
-  int wrapper_final_add_index(handler_add_index *add, bool commit);
-  int storage_final_add_index(handler_add_index *add, bool commit);
-#  endif
   int wrapper_prepare_drop_index(TABLE *table_arg, uint *key_num,
                                  uint num_of_keys);
   int storage_prepare_drop_index(TABLE *table_arg, uint *key_num,
@@ -1807,13 +1789,11 @@ private:
   bool wrapper_has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT;
   bool storage_has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT;
 #endif
+#ifdef MRN_HANDLER_HAVE_PRIMARY_KEY_IS_CLUSTERED
   bool wrapper_primary_key_is_clustered()
     MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST;
   bool storage_primary_key_is_clustered()
     MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST;
-#ifdef MRN_HANDLER_HAVE_GET_TABLESPACE_NAME
-  char *wrapper_get_tablespace_name(THD *thd, char *name, uint name_len);
-  char *storage_get_tablespace_name(THD *thd, char *name, uint name_len);
 #endif
 #ifdef MRN_HANDLER_HAVE_CAN_SWITCH_ENGINES
   bool wrapper_can_switch_engines();
@@ -1837,9 +1817,6 @@ private:
   void storage_init_table_handle_for_HANDLER();
   void wrapper_set_keys_in_use();
   void storage_set_keys_in_use();
-#ifdef MRN_RBR_UPDATE_NEED_ALL_COLUMNS
-  bool check_written_by_row_based_binlog();
-#endif
 #ifdef MRN_HAVE_HA_REBIND_PSI
   void wrapper_unbind_psi();
   void storage_unbind_psi();
