@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2015-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2015-2020 Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,56 +24,96 @@
 #  include <my_byteorder.h>
 #endif
 
-#if MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
-#  define MRN_DEST_IS_POINTER
-#endif
-
 namespace mrn {
+  namespace {
+#if MYSQL_VERSION_ID >= 80019 && !defined(MRN_MARIADB_P)
+    inline uint16 mrn_ushortget(const uchar *source) {
+      return ushortget(source);
+    }
+
+    inline float mrn_float4get(const uchar *source) {
+      return float4get(source);
+    }
+
+    inline double mrn_float8get(const uchar *source) {
+      return float8get(source);
+    }
+
+    inline long long int mrn_longlongget(const uchar *source) {
+      return longlongget(source);
+    }
+#elif MYSQL_VERSION_ID >= 50706 && !defined(MRN_MARIADB_P)
+    inline uint16 mrn_ushortget(const uchar *source) {
+      uint16 value;
+      ushortget(&value, source);
+      return value;
+    }
+
+    inline float mrn_float4get(const uchar *source) {
+      float value;
+      float4get(&value, source);
+      return value;
+    }
+
+    inline double mrn_float8get(const uchar *source) {
+      double value;
+      float8get(&value, source);
+      return value;
+    }
+
+    inline long long int mrn_longlongget(const uchar *source) {
+      long long int value;
+      longlongget(&value, source);
+      return value;
+    }
+#else
+    inline uint16 mrn_ushortget(const uchar *source) {
+      uint16 value;
+      ushortget(value, source);
+      return value;
+    }
+
+    inline float mrn_float4get(const uchar *source) {
+      float value;
+      float4get(value, source);
+      return value;
+    }
+
+    inline double mrn_float8get(const uchar *source) {
+      double value;
+      float8get(value, source);
+      return value;
+    }
+
+    inline long long int mrn_longlongget(const uchar *source) {
+      long long int value;
+      longlongget(value, source);
+      return value;
+    }
+#endif
+  }
+
   namespace value_decoder {
     void decode(uint16 *dest, const uchar *source) {
       MRN_DBUG_ENTER_FUNCTION();
-#ifdef MRN_DEST_IS_POINTER
-      ushortget(dest, source);
-#else
-      uint16 value;
-      ushortget(value, source);
-      *dest = value;
-#endif
+      *dest = mrn_ushortget(source);
       DBUG_VOID_RETURN;
     };
 
     void decode(float *dest, const uchar *source) {
       MRN_DBUG_ENTER_FUNCTION();
-#ifdef MRN_DEST_IS_POINTER
-      float4get(dest, source);
-#else
-      float value;
-      float4get(value, source);
-      *dest = value;
-#endif
+      *dest = mrn_float4get(source);
       DBUG_VOID_RETURN;
     };
 
     void decode(double *dest, const uchar *source) {
       MRN_DBUG_ENTER_FUNCTION();
-#ifdef MRN_DEST_IS_POINTER
-      float8get(dest, source);
-#else
-      double value;
-      float8get(value, source);
-      *dest = value;
-#endif
+      *dest = mrn_float8get(source);
       DBUG_VOID_RETURN;
     }
     void decode(long long int *dest, const uchar *source) {
       MRN_DBUG_ENTER_FUNCTION();
-#ifdef MRN_DEST_IS_POINTER
-      longlongget(dest, source);
-#else
-      long long int value;
-      longlongget(value, source);
-      *dest = value;
-#endif
+      *dest = mrn_longlongget(source);
       DBUG_VOID_RETURN;
     }
   }
